//
// Created by kirdyk on 20.10.16.
//

#include "certcheck.h"
#include "../sslclient.h"
#include "../utils.h"

namespace checks
{
    static const std::string certcheck_error_string = "Certificate check Error: ";
    
    CheckResult CertificateCheck::execute(ParamsMap const &params)
    {
        try
        {
            WOAP2_LOG_TRACE(" > CertificateCheck::execute")
    
            auto stringParamAccessor = params_accessor<std::string>();
            std::string _host = stringParamAccessor.get_required_param(params, host_param_name, certcheck_error_string);
    
            // default to port 443
            std::string host = _host, port = "443";
    
            // Find protocol part of the url
            size_t proto_pos = host.find("://");
            if(proto_pos != std::string::npos)
            {
                // Skip protocol part
                host = host.substr(proto_pos + 3);
            }
    
            // find path part of the URL
            size_t path_pos = host.find("/");
            if(path_pos != std::string::npos)
            {
                // Skip path path
                host = host.substr(0, path_pos);
            }
    
            // Check if host contain port
            if(host.find(':') != std::string::npos)
            {
                boost::regex colon_re(":");
                std::vector<std::string> host_parts = utils::split(host, colon_re);
        
                if(host_parts.size() > 1)
                {
                    host = host_parts[0];
                    port = host_parts[1];
                }
            }
    
            auto intParamAccessor = params_accessor<int>();
            int limit = intParamAccessor.get_required_param(params, limit_param_name, certcheck_error_string);
    
            WOAP2_LOG_DEBUG(std::string("Checking if certificate for ") +
                            _host + " is valid for " + std::to_string(limit) + " days");
    
            // initialize boost::asio stuff
            boost::asio::io_service io_service;
    
            boost::asio::ip::tcp::resolver resolver(io_service);
            boost::asio::ip::tcp::resolver::query query(host, port);
            boost::asio::ip::tcp::resolver::iterator iterator = resolver.resolve(query);
    
            boost::asio::ssl::context ctx(boost::asio::ssl::context::sslv23);
    
            // SSL checker client
            ssl::client ssl_client(limit, io_service, ctx, iterator);
    
            // Run boost::asio event loop
            io_service.run();
    
            WOAP2_LOG_TRACE(" < CertificateCheck::execute")
            return CheckResult{
                    !ssl_client.is_failed(),
                    ssl_client.is_still_valid(),
                    !ssl_client.is_failed() ? ssl_client.get_message() : "",
                    ssl_client.is_failed() ? ssl_client.get_message() : ""};
        }
        catch(boost::system::system_error error)
        {
            WOAP2_LOG_ERROR(error.code().message());
            WOAP2_LOG_TRACE(" < CertificateCheck::execute");
            return CheckResult{false, false, "", error.code().message()};
        }
        catch(std::runtime_error error)
        {
            WOAP2_LOG_ERROR(error.what());
            WOAP2_LOG_TRACE(" < CertificateCheck::execute");
            return CheckResult{false, false, "", error.what()};
        }
        catch(...)
        {
            WOAP2_LOG_ERROR("Unexpected exception");
            WOAP2_LOG_TRACE(" < CertificateCheck::execute");
            return CheckResult{false, false, "", "Unexpected exception"};
        }
    }
}
