//
// Created by kirdyk on 20.10.16.
//

#ifndef WOAP2_CERTCHECK_H
#define WOAP2_CERTCHECK_H


#include <string>
#include "check.h"
#include "../logger.h"

namespace checks
{
    /***
     * SSL Certificate check
     */
    class CertificateCheck : public CheckBase
    {
    private:
        static constexpr char const *const host_param_name = "Host name";
        static constexpr char const *const limit_param_name = "Remaining limit [days]";
    public:
        /***
         * Constructor.
         * @param for_params Just fill the parameters list
         */
        CertificateCheck(bool for_params)
        {
            WOAP2_LOG_TRACE(" > CertificateCheck::CertificateCheck")
            WOAP2_LOG_DEBUG(std::string("CertificateCheck ") + (for_params ? "for params" : "for check"))
            
            if(for_params)
            {
                check_params.push_back(CheckParameter{host_param_name, "text", 100});
                check_params.push_back(CheckParameter{limit_param_name, "longinteger", 1000});
            }
    
            WOAP2_LOG_TRACE(" < CertificateCheck::CertificateCheck")
        }
    
        /***
         * Destructor
         */
        virtual ~CertificateCheck()
        {
            WOAP2_LOG_TRACE(" > CertificateCheck::~CertificateCheck")
            WOAP2_LOG_TRACE(" < CertificateCheck::~CertificateCheck")
        }
    
        /***
         * Get check name
         * @return "Certificate"
         */
        static std::string get_name()
        {
            return "Certificate";
        }
    
        /***
         * Get check description
         * @return
         */
        virtual std::string get_description() override
        {
            return "Certificate validity test";
        }
    
        /***
         * Execute the check
         * @param params check parameters
         * @return result
         */
        virtual CheckResult execute(ParamsMap const &params) override;
    };
}

#endif //WOAP2_CERTCHECK_H
