//
// Created by kirdyk on 13.09.16.
//

#include "cpucheck.h"
#include <cstdio>
#include <vector>
#include <memory>
#include <boost/regex.hpp>
#include "../utils.h"

namespace checks
{
    static const std::string cpucheck_error_string = "CPU Check Error: ";
    
    CheckResult CpuCheck::execute(ParamsMap const &params)
    {
        WOAP2_LOG_TRACE(" > CpuCheck::execute")
    
        try
        {
            // Get parameters
            auto intParamAccessor = params_accessor<int>();
            int limit = intParamAccessor.get_required_param(params, limit_param_name, cpucheck_error_string);
    
            WOAP2_LOG_DEBUG(std::string("Executing CPU Check with load limit ") + std::to_string(limit));
    
            char buf[256];
    
            // Create a process
            WOAP2_LOG_TRACE("starting vmstat")
            auto proc = std::shared_ptr<FILE>(popen("vmstat 1 2", "r"), pclose);
    
            // skip headers
            fgets(buf, 255, proc.get());
            fgets(buf, 255, proc.get());
    
            // split header string and find positions of "sy" and "us" columns (system and user cpu usage)
            std::string header(buf);
            boost::regex whitespace("\\s+");
    
            std::vector<std::string> headers = utils::split(utils::trim(header), whitespace);
    
            int sy_idx = -1;
            int us_idx = -1;
            int i = 0;
    
            for(std::vector<std::string>::const_iterator it = headers.begin(); it != headers.end(); ++it, i++)
            {
                if(*it == "sy")
                {
                    sy_idx = i;
                }
                else if(*it == "us")
                {
                    us_idx = i;
                }
            }
    
            WOAP2_LOG_TRACE(
                    std::string("sy index is ") + std::to_string(sy_idx) + " and us index is " + std::to_string(us_idx))
    
            // Get the second data string.
            fgets(buf, 255, proc.get());
            fgets(buf, 255, proc.get());
    
            // split the data string and get usage values
            std::string data(buf);
            std::vector<std::string> data_vec = utils::split(utils::trim(data), whitespace);
    
            int sy_load = std::stoi(data_vec[sy_idx]);
            int us_load = std::stoi(data_vec[us_idx]);
    
            WOAP2_LOG_TRACE(
                    std::string("sy load is ") + std::to_string(sy_load) + " and us load is " + std::to_string(us_load))
    
            // construct the result
            int load = sy_load + us_load;
            bool under_limit = (load <= limit);
    
            std::string result = "CPU load " +
                                 ((under_limit) ? "is " + std::to_string(load)
                                                : std::to_string(load) + " out of Limit " + std::to_string(limit));
    
            WOAP2_LOG_TRACE(" < CpuCheck::execute")
            return CheckResult{true, under_limit, result, ""};
        }
        catch(std::runtime_error error)
        {
            WOAP2_LOG_ERROR(error.what());
            WOAP2_LOG_TRACE(" < CpuCheck::execute");
            return CheckResult{false, false, "", error.what()};
        }
        catch(...)
        {
            WOAP2_LOG_ERROR("Unexpected exception");
            WOAP2_LOG_TRACE(" < CpuCheck::execute");
            return CheckResult{false, false, "", "Unexpected exception"};
        }
    }
}