//
// Created by kirdyk on 13.09.16.
//

#ifndef WOAP2_CPUCHECK_H
#define WOAP2_CPUCHECK_H

#include <string>
#include "check.h"
#include "../logger.h"

namespace checks
{
    /**
     * CPU Check
     */
    class CpuCheck : public CheckBase
    {
    private:
        // Parameters' names
        static constexpr char const *const limit_param_name = "Load Limit [%]";
    public:
        /**
         * Constructor. Fills parameter list.
         */
        CpuCheck(bool for_params)
        {
            WOAP2_LOG_TRACE(" > CpuCheck::CpuCheck")
            WOAP2_LOG_DEBUG(std::string("CpuCheck ") + (for_params ? "for params" : "for check"))
    
            if(for_params)
            {
                check_params.push_back(CheckParameter{limit_param_name, "longinteger", 200});
            }
    
            WOAP2_LOG_TRACE(" < CpuCheck::CpuCheck")
        }
    
        /***
         * Destructor
         */
        virtual ~CpuCheck()
        {
            WOAP2_LOG_TRACE(" > CpuCheck::~CpuCheck")
            WOAP2_LOG_TRACE(" < CpuCheck::~CpuCheck")
        }
        
        /**
         * Get check name.
         * @return "CPU" as check name.
         */
        static std::string get_name()
        {
            return "CPU";
        }
    
        /**
         * Get check description.
         * @return "CPU Usage" as a description.
         */
        virtual std::string get_description() override
        {
            return "CPU Usage";
        }
    
        /**
         * Execute the check.
         * @param params check parameters.
         * @return check result.
         */
        virtual CheckResult execute(ParamsMap const &params) override;
    };
    
}
#endif //WOAP2_CPUCHECK_H
