//
// Created by kirdyk on 13.09.16.
//

#include "diskspacecheck.h"
#include <memory>
#include <sys/types.h>
#include <sys/stat.h>
#include <boost/regex.hpp>
#include "../utils.h"

namespace checks
{
    static const std::string diskcheck_error_string = "Disk space check Error: ";
    
    std::shared_ptr<FILE> DiskSpaceCheck::exec_df() const
    {
        WOAP2_LOG_TRACE(" > DiskSpaceCheck::exec_df")
        
        struct stat df_stat;
        
        std::string cmd = "df -P";
        
        if(!stat("/usr/xpg4/bin/df", &df_stat))
        {
            cmd = "/usr/xpg4/bin/df -P";
        }
    
        WOAP2_LOG_TRACE(std::string("Executing ") + cmd)
        
        // Create a process
        auto proc = std::shared_ptr<FILE>(popen(cmd.c_str(), "r"), pclose);
    
        WOAP2_LOG_TRACE(" < DiskSpaceCheck::exec_df")
        return proc;
    }
    
    std::vector<std::string> DiskSpaceCheck::get_disks() const
    {
        WOAP2_LOG_TRACE(" > DiskSpaceCheck::get_disks")
        
        std::vector<std::string> result;
        
        char buf[256];
        boost::regex whitespace("\\s+");
        
        std::shared_ptr<FILE> proc = exec_df();
        
        // skip header
        fgets(buf, 255, proc.get());
        
        // get data
        while(fgets(buf, 255, proc.get()))
        {
            std::string data_string(buf);
            
            std::vector<std::string> data_vec = utils::split(utils::trim(data_string), whitespace);
            
            std::vector<std::string>::const_reverse_iterator mount_it = data_vec.rbegin();
            
            result.push_back(*mount_it);
        }
    
        WOAP2_LOG_TRACE(" < DiskSpaceCheck::get_disks")
        
        return result;
    }
    
    CheckResult DiskSpaceCheck::execute(ParamsMap const &params)
    {
        try
        {
            WOAP2_LOG_TRACE(" > DiskSpaceCheck::execute")
    
            // Get parameters
            auto intParamAccessor = params_accessor<int>();
            int limit = intParamAccessor.get_required_param(params, limit_param_name, diskcheck_error_string);
    
            auto stringParamAccessor = params_accessor<std::string>();
            std::string disk = stringParamAccessor.get_required_param(params, disks_param_name, diskcheck_error_string);
    
            WOAP2_LOG_DEBUG(std::string("Executing disk space check for ") + disk + " with free limit " +
                            std::to_string(limit));
    
            // start df process
            char buf[256];
            boost::regex whitespace("\\s+");
    
            std::shared_ptr<FILE> proc = exec_df();
    
            // skip header
            fgets(buf, 255, proc.get());
    
            std::string usage_str = "100%";
            int usage = 100;
    
            // get data
            while(fgets(buf, 255, proc.get()))
            {
                std::string data_string(buf);
        
                std::vector<std::string> data_vec = utils::split(utils::trim(data_string), whitespace);
        
                std::vector<std::string>::const_reverse_iterator data_it = data_vec.rbegin();
        
                // If the last part contains disk, then the part before contains usage.
                if(*data_it == disk)
                {
                    data_it++;
                    usage_str = data_it->substr(0, data_it->length() - 1);
                    usage = std::stoi(usage_str);
                    break;
                }
            }
    
            // Skip other data
            while(fgets(buf, 255, proc.get()))
            {
            }
    
            bool under_limit = (usage <= limit);
    
            WOAP2_LOG_TRACE(std::string("Disk ") + disk + "usage is " + std::to_string(usage) + "%")
    
            std::string result = (under_limit) ? "Disk " + disk + " is " + usage_str + "% used"
                                               : "Out of disk space on disk " + disk + ": " + usage_str + "% used";
    
            WOAP2_LOG_TRACE(" < DiskSpaceCheck::execute")
    
            return CheckResult{true, under_limit, result, ""};
        }
        catch(std::runtime_error error)
        {
            WOAP2_LOG_ERROR(error.what());
            WOAP2_LOG_TRACE(" < DiskSpaceCheck::execute");
            return CheckResult{false, false, "", error.what()};
        }
        catch(...)
        {
            WOAP2_LOG_ERROR("Unexpected exception");
            WOAP2_LOG_TRACE(" < DiskSpaceCheck::execute");
            return CheckResult{false, false, "", "Unexpected exception"};
        }
    }
}