//
// Created by kirdyk on 13.09.16.
//

#ifndef WOAP2_DISKSPACECHECK_H
#define WOAP2_DISKSPACECHECK_H

#include <string>
#include "check.h"
#include "../logger.h"

namespace checks
{
    /***
     * Disk free space check/
     */
    class DiskSpaceCheck : public CheckBase
    {
    private:
        // parameters' names
        static constexpr char const *const limit_param_name = "Disk Free Limit [%]";
        static constexpr char const *const disks_param_name = "Disks";
    
        /***
         * Execute df
         * @return pointer to stdout file object.
         */
        std::shared_ptr<FILE> exec_df() const;
    
        /***
         * Enumerate mount points
         * @return array of mount points
         */
        std::vector<std::string> get_disks() const;
    
    public:
    
        /***
         * Constructor
         * @param for_params just fill the check parameters description
         */
        DiskSpaceCheck(bool for_params)
        {
            WOAP2_LOG_TRACE(" > DiskSpaceCheck::DiskSpaceCheck")
            WOAP2_LOG_DEBUG(std::string("DiskSpaceCheck ") + (for_params ? "for params" : "for check"))
    
            if(for_params)
            {
                check_params.push_back(CheckParameter{limit_param_name, "longinteger", 100});
                check_params.push_back(CheckParameter{disks_param_name, "listcombo", 0});
    
                combo_values_map[disks_param_name] = get_disks();
            }
    
            WOAP2_LOG_TRACE(" < DiskSpaceCheck::DiskSpaceCheck")
        }
    
        /***
         * Destructor
         */
        virtual ~DiskSpaceCheck()
        {
            WOAP2_LOG_TRACE(" > DiskSpaceCheck::~DiskSpaceCheck")
            WOAP2_LOG_TRACE(" < DiskSpaceCheck::~DiskSpaceCheck")
        }
    
        /***
         * Get name of the check
         * @return "DiskSpace"
         */
        static std::string get_name()
        {
            return "DiskSpace";
        }
    
        /***
         * Get check description
         * @return
         */
        virtual std::string get_description() override
        {
            return "DiskSpace Usage";
        }
    
        /***
         * Execute the check
         * @param params check parameters
         * @return result
         */
        virtual CheckResult execute(ParamsMap const &params) override;
    };
    
}
#endif //WOAP2_DISKSPACECHECK_H
