//
// Created by kirdyk on 14.09.16.
//

#include "memorycheck.h"
#include <memory>
#include <boost/regex.hpp>
#include "../utils.h"

namespace checks
{
    static const std::string ramcheck_error_string = "RAM usage check Error: ";
    static const std::string swapcheck_error_string = "Swap usage check Error: ";
    
    std::pair<int, int> MemoryCheck::get_memory_usage() const
    {
        WOAP2_LOG_TRACE(" > MemoryCheck::get_memory_usage")
        
        char buf[2048];
        
        // Create a process
        WOAP2_LOG_TRACE("Executing top")
        
        auto proc = std::shared_ptr<FILE>(popen("top -d 1 -n 1 -b", "r"), pclose);
        
        int mem_usage = 0, swap_usage = 0;
        bool mem_found = false, swap_found = false;
        
        boost::regex not_num_re("(\\D|\\s)+");
        boost::regex whitespace_re("\\s+");
        
        // skip headers
        while((!mem_found || !swap_found) && fgets(buf, 2048, proc.get()))
        {
            std::string data(buf);
            data = utils::trim(data);
            
            std::string nums_str = utils::trim(boost::regex_replace(data, not_num_re, " "));
            std::vector<std::string> nums_strs = utils::split(nums_str, whitespace_re);
            std::vector<long> nums;
            nums.reserve(nums_strs.size());
            
            for(std::vector<std::string>::const_iterator it = nums_strs.begin(); it != nums_strs.end(); ++it)
            {
                nums.push_back(std::stol(*it));
            }
            
            int nums_count = (int) nums.size();
            
            // All magic was taken from woap v1.
            // It didn't work on moders linuxes.
            // Memory lines didn't start with "Mem:" or "Swap:" but "KiB Mem:".
            // I have replaced strncmp(buff,"Mem:",4) with substring search.
            type mem_type = NONE;
            
            if(data.find("Mem:") != std::string::npos)
            {
                WOAP2_LOG_TRACE("Mem: found")
                mem_type = MEM;
            }
            else if(data.find("Swap:") != std::string::npos)
            {
                WOAP2_LOG_TRACE("Swap: found")
                mem_type = SWAP;
            }
            else if(data.find("Memory:") != std::string::npos)
            {
                WOAP2_LOG_TRACE("Memory: found")
                mem_type = MEMORY;
            }
            
            if(data.find("Memory: Real:") != std::string::npos)
            {
                WOAP2_LOG_TRACE("Memory: Real: found")
                WOAP2_LOG_TRACE(std::string("\tdata string was : ") + data)
                
                mem_usage = get_usage_percent(nums[1], nums[1] + nums[2]);
                swap_usage = get_usage_percent(nums[3], nums[4]);
                
                mem_found = swap_found = true;
                
                continue;
            }
            
            if(mem_type == NONE)
            {
                continue;
            }
            
            if(mem_type == MEM && (nums_count == 4 || nums_count == 5))
            {
                WOAP2_LOG_TRACE("MEM && count in [4,5]")
                WOAP2_LOG_TRACE(std::string("\tdata string was : ") + data)
                
                mem_usage = get_usage_percent(nums[1], nums[0]);
                mem_found = true;
            }
            else if(mem_type == MEM && nums_count == 6)
            {
                WOAP2_LOG_TRACE("MEM && count is 6")
                WOAP2_LOG_TRACE(std::string("\tdata string was : ") + data)
                
                long mem_used = nums[0] + nums[2] + nums[4];
                long mem_free = nums[1] + nums[3] + nums[5];
                mem_usage = get_usage_percent(mem_used, mem_used + mem_free);
                mem_found = true;
            }
            else if(mem_type == SWAP && nums_count == 4)
            {
                WOAP2_LOG_TRACE("SWAP && count is 4")
                WOAP2_LOG_TRACE(std::string("\tdata string was : ") + data)
                
                swap_usage = get_usage_percent(nums[1], nums[0]);
                swap_found = true;
            }
            else if(mem_type == SWAP && nums_count == 2)
            {
                WOAP2_LOG_TRACE("SWAP && count is 2")
                WOAP2_LOG_TRACE(std::string("\tdata string was : ") + data)
                
                swap_usage = get_usage_percent(nums[0] - nums[1], nums[0]);
                swap_found = true;
            }
            else if(mem_type == MEMORY && nums_count == 5)
            {
                WOAP2_LOG_TRACE("MEMORY && count is 5")
                WOAP2_LOG_TRACE(std::string("\tdata string was : ") + data)
                
                mem_usage = get_usage_percent(nums[0] - nums[2], nums[0]);
                swap_usage = get_usage_percent(nums[3] - nums[4], nums[3]);
                mem_found = swap_found = true;
            }
            else if(mem_type == MEMORY && nums_count == 4)
            {
                WOAP2_LOG_TRACE("MEMORY && count is 4")
                WOAP2_LOG_TRACE(std::string("\tdata string was : ") + data)
                
                mem_usage = get_usage_percent(nums[0] - nums[1], nums[0]);
                swap_usage = get_usage_percent(nums[2] - nums[3], nums[2]);
                mem_found = swap_found = true;
            }
            else if(mem_type == MEM)
            {
                WOAP2_LOG_WARN("Unknown MEM format")
                WOAP2_LOG_DEBUG(std::string("\tdata string was : ") + data)
            }
            else if(mem_type == SWAP)
            {
                WOAP2_LOG_WARN("Unknown SWAP format")
                WOAP2_LOG_DEBUG(std::string("\tdata string was : ") + data)
            }
            else if(mem_type == MEMORY)
            {
                WOAP2_LOG_WARN("Unknown MEMORY format")
                WOAP2_LOG_DEBUG(std::string("\tdata string was : ") + data)
            }
        }
        
        // skip other data
        while(fgets(buf, 2048, proc.get()))
        {
        }
    
        WOAP2_LOG_TRACE(" < MemoryCheck::get_memory_usage")
        return std::pair<int, int>{mem_usage, swap_usage};
    }
    
    CheckResult RamCheck::execute(ParamsMap const &params)
    {
        try
        {
            WOAP2_LOG_TRACE(" > RamCheck::execute")
    
            // get params
            auto intParamAccessor = params_accessor<int>();
            int limit = intParamAccessor.get_required_param(params, limit_param_name, ramcheck_error_string);
    
            WOAP2_LOG_DEBUG(std::string("Executing RAM usage check with free limit") + std::to_string(limit));
    
            int free_mem = get_memory_usage().first;
    
            bool enough_mem = (free_mem > limit);
    
            WOAP2_LOG_TRACE(std::string("Free RAM percentage is ") + std::to_string(free_mem) + "%")
    
            std::string result = (enough_mem) ? std::to_string(free_mem) + "% of RAM memory free"
                                              : "Low RAM memory: " + std::to_string(free_mem) + "% free";
    
            WOAP2_LOG_TRACE(" < RamCheck::execute")
            return CheckResult{true, enough_mem, result, ""};
        }
        catch(std::runtime_error error)
        {
            WOAP2_LOG_ERROR(error.what());
            WOAP2_LOG_TRACE(" < RamCheck::execute");
            return CheckResult{false, false, "", error.what()};
        }
        catch(...)
        {
            WOAP2_LOG_ERROR("Unexpected exception");
            WOAP2_LOG_TRACE(" < RamCheck::execute");
            return CheckResult{false, false, "", "Unexpected exception"};
        }
    }
    
    CheckResult SwapCheck::execute(ParamsMap const &params)
    {
        try
        {
            WOAP2_LOG_TRACE(" > SwapCheck::execute")
    
            // get params
            auto intParamAccessor = params_accessor<int>();
            int limit = intParamAccessor.get_required_param(params, limit_param_name, swapcheck_error_string);
    
            WOAP2_LOG_DEBUG(std::string("Executing Swap usage check with free limit ") + std::to_string(limit));
    
            int free_mem = get_memory_usage().second;
    
            bool enough_mem = (free_mem > limit);
    
            WOAP2_LOG_TRACE(std::string("Free Swap percentage is ") + std::to_string(free_mem) + "%")
    
            std::string result = (enough_mem) ? std::to_string(free_mem) + "% of SWAP memory free"
                                              : "Low SWAP memory: " + std::to_string(free_mem) + "% free";
    
            WOAP2_LOG_TRACE(" < SwapCheck::execute")
            return CheckResult{true, enough_mem, result, ""};
        }
        catch(std::runtime_error error)
        {
            WOAP2_LOG_ERROR(error.what());
            WOAP2_LOG_TRACE(" < SwapCheck::execute");
            return CheckResult{false, false, "", error.what()};
        }
        catch(...)
        {
            WOAP2_LOG_ERROR("Unexpected exception");
            WOAP2_LOG_TRACE(" < SwapCheck::execute");
            return CheckResult{false, false, "", "Unexpected exception"};
        }
    }
}