//
// Created by kirdyk on 14.09.16.
//

#ifndef WOAP2_MEMORYCHECK_H
#define WOAP2_MEMORYCHECK_H

#include <string>
#include "check.h"
#include "../logger.h"

namespace checks
{
    /***
     * Base memory check class
     */
    class MemoryCheck : public CheckBase
    {
        /***
         * Check swap
         */
        bool swap;
    
        /***
         * top record type
         */
        enum type
        {
            NONE,
            MEM,
            SWAP,
            MEMORY,
        };
    
        /***
         * Calculate usage percentage
         * @param usage memory usage
         * @param total memory amount
         * @return
         */
        int get_usage_percent(long usage, long total) const
        {
            return (int) ((total) ? 100 - (100 * usage / total) : 100);
        }
    
    protected:
        /***
         * Get RAM and swap usage
         * @return
         */
        std::pair<int, int> get_memory_usage() const;
    
    public:
        /***
         * Constructor
         * @param swap check for swap
         */
        MemoryCheck(bool swap)
        {
            WOAP2_LOG_TRACE(" > MemoryCheck::MemoryCheck")
            WOAP2_LOG_DEBUG(std::string("MemoryCheck for ") + (swap ? "Swap" : "RAM"))
            
            this->swap = swap;
    
            WOAP2_LOG_TRACE(" < MemoryCheck::MemoryCheck")
        }
    
        /***
         * Destructor
         */
        virtual ~MemoryCheck()
        {
            WOAP2_LOG_TRACE(" > MemoryCheck::MemoryCheck")
            WOAP2_LOG_TRACE(" < MemoryCheck::MemoryCheck")
        }
    };
    
    /***
     * RAM check
     */
    class RamCheck : public MemoryCheck
    {
    private:
        // parameters' names
        static constexpr char const *const limit_param_name = "Free RAM Limit [%]";
    public:
        /***
         * Constructor
         * @param for_params just fill parameters description
         */
        RamCheck(bool for_params)
                : MemoryCheck(false)
        {
            WOAP2_LOG_TRACE(" > RamCheck::RamCheck")
            WOAP2_LOG_DEBUG(std::string("RamCheck ") + (for_params ? "for params" : "for check"))
    
            if(for_params)
            {
                check_params.push_back(CheckParameter{limit_param_name, "longinteger", 100});
            }
            WOAP2_LOG_TRACE(" < RamCheck::RamCheck")
        }
    
        /***
         * Destructor
         */
        virtual ~RamCheck()
        {
            WOAP2_LOG_TRACE(" > RamCheck::~RamCheck")
            WOAP2_LOG_TRACE(" < RamCheck::~RamCheck")
        }
    
        /***
         * Get name of the check
         * @return "RAM"
         */
        static std::string get_name()
        {
            return "RAM";
        }
    
        /***
         * Get check description
         * @return
         */
        virtual std::string get_description() override
        {
            return "RAM Usage";
        }
    
        /***
         * Execute check
         * @param params check parameters
         * @return result
         */
        virtual CheckResult execute(ParamsMap const &params) override;
        
    };
    
    /***
     * Swap check
     */
    class SwapCheck : public MemoryCheck
    {
    private:
        // parameters' names
        static constexpr char const *const limit_param_name = "Free Swap Limit [%]";
        
    public:
        /***
         * Constructor
         * @param for_params just fill the parameters description
         */
        SwapCheck(bool for_params)
                : MemoryCheck(true)
        {
            WOAP2_LOG_TRACE(" > SwapCheck::SwapCheck")
            WOAP2_LOG_DEBUG(std::string("SwapCheck ") + (for_params ? "for params" : "for check"))
    
            if(for_params)
            {
                check_params.push_back(CheckParameter{limit_param_name, "longinteger", 100});
            }
    
            WOAP2_LOG_TRACE(" < SwapCheck::SwapCheck")
        }
    
        /***
         * Destructor
         */
        virtual ~SwapCheck()
        {
            WOAP2_LOG_TRACE(" > SwapCheck::~SwapCheck")
            WOAP2_LOG_TRACE(" < SwapCheck::~SwapCheck")
        }
    
        /***
         * Get the name of the check
         * @return "Swap"
         */
        static std::string get_name()
        {
            return "Swap";
        }
    
        /***
         * Get the description of the check
         * @return
         */
        virtual std::string get_description() override
        {
            return "Swap Usage";
        }
    
        /***
         * Execute the check
         * @param params check parameters
         * @return result
         */
        virtual CheckResult execute(ParamsMap const &params) override;
        
    };
    
}

#endif //WOAP2_MEMORYCHECK_H
