//
// Created by kirdyk on 14.10.16.
//

#ifndef WOAP2_PINGCHECK_H
#define WOAP2_PINGCHECK_H

#include <string>
#include "check.h"
#include "../logger.h"

namespace checks
{
    /***
     * Ping check
     */
    class PingCheck : public CheckBase
    {
    private:
        // parameters' names
        static constexpr char const *const host_param_name = "Host name or IP";
        static constexpr char const *const ipv6_param_name = "Use IPv6";
        static constexpr char const *const pkts_num_param_name = "Number of packets";
        static constexpr char const *const timeout_param_name = "Overall timeout [s]";
        static constexpr char const *const limit_param_name = "Success rate limit [%]";
        /***
         * Perform ping
         * @param host host to ping
         * @param ipv6 use IPv6
         * @return round-trip time of the ping or -1 if ping failed
         */
        double perform_ping(std::string &host, bool ipv6) const;
        
    public:
        /***
         * Constructor
         * @param for_params just fill the parameters description
         */
        PingCheck(bool for_params)
        {
            WOAP2_LOG_TRACE(" > PingCheck::PingCheck")
            WOAP2_LOG_DEBUG(std::string("PingCheck ") + (for_params ? "for params" : "for check"))
            
            if(for_params)
            {
                check_params.push_back(CheckParameter{host_param_name, "text", 100});
                check_params.push_back(CheckParameter{ipv6_param_name, "boolean", 1});
                check_params.push_back(CheckParameter{pkts_num_param_name, "longinteger", 20});
                check_params.push_back(CheckParameter{timeout_param_name, "longinteger", 100});
                check_params.push_back(CheckParameter{limit_param_name, "longinteger", 100});
            }
    
            WOAP2_LOG_TRACE(" < PingCheck::PingCheck")
        }
    
        /***
         * Destructor
         */
        virtual ~PingCheck()
        {
            WOAP2_LOG_TRACE(" > PingCheck::~PingCheck")
            WOAP2_LOG_TRACE(" < PingCheck::~PingCheck")
        }
    
        /***
         * Get the name of the check
         * @return "Ping"
         */
        static std::string get_name()
        {
            return "Ping";
        }
    
        /***
         * Get the description of the check
         * @return
         */
        virtual std::string get_description() override
        {
            return "Host or IP ping test";
        }
    
        /***
         * Execute the check
         * @param params check parameters
         * @return result
         */
        virtual CheckResult execute(ParamsMap const &params) override;
    };
    
}
#endif //WOAP2_PINGCHECK_H
