//
// Created by kirdyk on 15.09.16.
//

#include "proccheck.h"
#include <memory>
#include <sys/utsname.h>
#include <boost/regex.hpp>
#include "../utils.h"

namespace checks
{
    static const std::string proccheck_error_string = "Process check Error: ";
    
    std::vector<std::string> ProcCheck::get_procs() const
    {
        WOAP2_LOG_TRACE(" > ProcCheck::get_procs")
        
        std::set<std::string> procs = get_procs_set();
        std::vector<std::string> result;
        
        result.reserve(procs.size());
        
        for(std::set<std::string>::const_iterator it = procs.begin(); it != procs.end(); ++it)
        {
            result.push_back(*it);
        }
    
        WOAP2_LOG_TRACE(" < ProcCheck::get_procs")
        return result;
    }
    
    std::set<std::string> ProcCheck::get_procs_set() const
    {
        WOAP2_LOG_TRACE(" > ProcCheck::get_procs_set")
        
        struct utsname machine;
        
        std::string cmd = "ps -ax -o user,args";
        
        uname(&machine);
        
        std::string os_name(machine.sysname);
        
        if(os_name == "IRIX64" || os_name == "SunOS")
        {
            cmd = "ps -e -o user,args";
        }
        
        // Create a process
        WOAP2_LOG_TRACE(std::string("Executing ") + cmd)
        auto proc = std::shared_ptr<FILE>(popen(cmd.c_str(), "r"), pclose);
        
        char buf[2048];
        boost::regex whitespace("\\s+");
        
        // skip header
        fgets(buf, 2048, proc.get());
        
        std::set<std::string> procs;
        boost::regex re("(\\S+)\\s+(.+)");
        
        // get data
        while(fgets(buf, 2048, proc.get()))
        {
            // TODO: Unicode support
            std::string data_string(buf);
            data_string = utils::trim(data_string);
            
            std::vector<std::string> parts;
            
            copy(boost::sregex_token_iterator(data_string.begin(), data_string.end(), re, {1, 2},
                                              boost::regex_constants::match_not_null),
                 boost::sregex_token_iterator(),
                 back_inserter(parts));
            
            if(parts.size() < 2)
            {
                continue;
            }
            
            auto user = parts[0];
            auto process = parts[1];
    
            // Skip root and woap processes
            if(user == "root" || user == "woap")
            {
                continue;
            }
    
            // Skip kernel processes
            if(process[0] != '[')
            {
                std::vector<std::string> process_parts = utils::split(process, whitespace);
                unsigned long slash_pos = process_parts[0].find_last_of("/");
                
                if(slash_pos != std::string::npos)
                {
                    process = process.substr(slash_pos + 1);
                }
            }
            
            procs.insert(process);
        }
    
        WOAP2_LOG_TRACE(" < ProcCheck::get_procs_set")
        return procs;
    }
    
    CheckResult ProcCheck::execute(ParamsMap const &params)
    {
        WOAP2_LOG_TRACE(" > ProcCheck::execute")
    
        try
        {
            // get parameters
            auto stringParamsAccessor = params_accessor<std::string>();
            std::string proc = stringParamsAccessor.get_required_param(params, procs_param_name,
                                                                       proccheck_error_string);
    
            WOAP2_LOG_DEBUG(std::string("Executing proocess check for process ") + proc);
    
            std::set<std::string> procs = get_procs_set();
    
            bool found = (procs.find(proc) != procs.end());
    
            WOAP2_LOG_TRACE(std::string("Process ") + proc + (found ? " found" : " not found"))
    
            std::string result = found ? "Process " + proc + " is running"
                                       : "Process " + proc + " is not running";
    
            WOAP2_LOG_TRACE(" < ProcCheck::execute")
            return CheckResult{true, found, result, ""};
        }
        catch(std::runtime_error &exception)
        {
            std::string err_string = exception.what();
            WOAP2_LOG_TRACE(" < ProcCheck::execute")
            return CheckResult{false, false, "", err_string};;
        }
        catch(...)
        {
            WOAP2_LOG_ERROR("Unexpected exception");
            WOAP2_LOG_TRACE(" < ProcCheck::execute")
            return CheckResult{false, false, "", "Unexpected exception"};
        }
    }
}