//
// Created by kirdyk on 15.09.16.
//

#ifndef WOAP2_PROCCHECK_H
#define WOAP2_PROCCHECK_H

#include <string>
#include <set>
#include "check.h"
#include "../logger.h"

namespace checks
{
    /***
     * Process check
     */
    class ProcCheck : public CheckBase
    {
    private:
        // parameters' names
        static constexpr char const *const procs_param_name = "Select Process";
        
        /***
         * Get the list of running processes.
         * @return array of processes names
         */
        std::vector<std::string> get_procs() const;
    
        /***
         * Get the set of the running processes.
         * Set filters duplicates and sorts items.
         * @return set of running processes names
         */
        std::set<std::string> get_procs_set() const;
    
    public:
    
        /***
         * Constructor
         * @param for_params just fill parameters description
         */
        ProcCheck(bool for_params)
        {
            WOAP2_LOG_TRACE(" > ProcCheck::ProcCheck")
            WOAP2_LOG_DEBUG(std::string("ProcCheck ") + (for_params ? "for params" : "for check"))
    
            if(for_params)
            {
                check_params.push_back(CheckParameter{procs_param_name, "listcombo", 0});
        
                std::vector<std::string> procs = get_procs();
                combo_values_map[procs_param_name] = procs;
            }
    
            WOAP2_LOG_TRACE(" < ProcCheck::ProcCheck")
        }
    
        /***
         * Destructor
         */
        virtual ~ProcCheck()
        {
            WOAP2_LOG_TRACE(" > ProcCheck::~ProcCheck")
            WOAP2_LOG_TRACE(" < ProcCheck::~ProcCheck")
        }
    
        /***
         * Get the name of the check
         * @return "Processes"
         */
        static std::string get_name()
        {
            return "Processes";
        }
    
        /***
         * Get the description of the check
         * @return
         */
        virtual std::string get_description() override
        {
            return "Running processes";
        }
    
        /***
         * Execute the check
         * @param params check parameters
         * @return result
         */
        virtual CheckResult execute(ParamsMap const &params) override;
    };
}

#endif //WOAP2_PROCCHECK_H
