//
// Created by kirdyk on 30.12.16.
//

#include "snmpcheck.h"
#include "../utils.h"
#include "../snmpclient.h"
#include "../constants.h"

namespace checks
{
    /**
     * Check error header string
     */
    static const std::string snmp_error_string = "SNMP check Error: ";
    
    CheckResult SnmpCheck::execute(ParamsMap const &params)
    {
        WOAP2_LOG_TRACE(" > SnmpCheck::execute")
    
        try
        {
            // Extract parameters
            auto stringParamsAccessor = params_accessor<std::string>();
            std::string host = stringParamsAccessor.get_required_param(params, host_param_name, snmp_error_string);
            std::string community = stringParamsAccessor.get_required_param(params, community_param_name,
                                                                            snmp_error_string);
            std::string oidStr = stringParamsAccessor.get_required_param(params, oid_param_name, snmp_error_string);
            std::string test_type = stringParamsAccessor.get_required_param(params, check_param_name,
                                                                            snmp_error_string);
            std::string test_value = stringParamsAccessor.get_required_param(params, test_value_param_name,
                                                                             snmp_error_string);
    
            WOAP2_LOG_DEBUG(std::string("Executing SNMP v2c check to ") + host + ":" + community +
                            ". Checking if " + oidStr + " " + test_type + " " + test_value);
            
            // Create SNMP V2c client
            snmp::SnmpClient client(host, community, test_value, test_type);
            // and perform the actual work.
            return client.get_value(oidStr);
        }
        catch(std::runtime_error &exception)
        {
            WOAP2_LOG_TRACE(" < SnmpCheck::execute")
            return CheckResult{false, false, "", (exception.what())};
        }
        catch(std::invalid_argument &exception)
        {
            WOAP2_LOG_TRACE(" < SnmpCheck::execute")
            return CheckResult{false, false, "", "Invalid argument"};
        }
        catch(...)
        {
            WOAP2_LOG_ERROR("Unexpected exception");
            WOAP2_LOG_TRACE(" < SnmpCheck::execute")
            return CheckResult{false, false, "", "Unexpected exception"};
        }
    }
}