//
// Created by kirdyk on 30.12.16.
//

#ifndef WOAP2_SNMPCHECK_H
#define WOAP2_SNMPCHECK_H

#include <string>
#include "check.h"
#include "../logger.h"
#include "../constants.h"

namespace checks
{
    /***
     * SNMP v2c check
     */
    class SnmpCheck : public CheckBase
    {
    private:
        // parameters' names
        static constexpr char const *const host_param_name = "Host address";
        static constexpr char const *const community_param_name = "Community";
        static constexpr char const *const oid_param_name = "OID";
        static constexpr char const *const check_param_name = "Test";
        static constexpr char const *const test_value_param_name = "Test value";
    public:
        /***
         * Constructor
         * @param for_params just fill parameters description
         */
        SnmpCheck(bool for_params)
        {
            WOAP2_LOG_TRACE(" > SnmpCheck::SnmpCheck")
            WOAP2_LOG_DEBUG(std::string("SnmpCheck ") + (for_params ? "for params" : "for check"))
            
            if(for_params)
            {
                check_params.push_back(CheckParameter{host_param_name, "text", 255});
                check_params.push_back(CheckParameter{community_param_name, "text", 255, "public"});
                check_params.push_back(CheckParameter{oid_param_name, "text", 255});
    
                check_params.push_back(CheckParameter{check_param_name, "listdrop", 0, constants::equal_string});
                combo_values_map[check_param_name] = std::vector<std::string>{
                        constants::less_string, constants::less_or_equal_string,
                        constants::equal_string, constants::not_equal_string,
                        constants::greater_or_equal_string, constants::greater_string,
                        constants::contains_string, constants::not_contains_string
                };
    
                check_params.push_back(CheckParameter{test_value_param_name, "text", 255});
            }
            
            WOAP2_LOG_TRACE(" < SnmpCheck::SnmpCheck")
        }
        
        /***
         * Destructor
         */
        virtual ~SnmpCheck()
        {
            WOAP2_LOG_TRACE(" > SnmpCheck::~SnmpCheck")
            WOAP2_LOG_TRACE(" < SnmpCheck::~SnmpCheck")
        }
        
        /***
         * Get the name of the check
         * @return "URL"
         */
        static std::string get_name()
        {
            return "SNMP";
        }
        
        /***
         * Get the description of the check
         * @return
         */
        virtual std::string get_description() override
        {
            return "SNMP v2c value check";
        }
        
        /***
         * Execute check
         * @param params check parameters
         * @return result
         */
        virtual CheckResult execute(ParamsMap const &params) override;
        
    };
}

#endif //WOAP2_SNMPCHECK_H
