//
// Created by bezgin on 30.11.16.
//

#ifndef WOAP2_URLCHECK_H
#define WOAP2_URLCHECK_H

#include <string>
#include "check.h"
#include "../logger.h"

namespace checks
{
    /***
     * URL HTTP(S) status check
     */
    class UrlCheck : public CheckBase
    {
    private:
        // parameters' names
        static constexpr char const *const url_param_name = "URL";
        static constexpr char const *const status_param_name = "Status";
        static constexpr char const *const search_text_param_name = "Text to find";
    public:
        /***
         * Constructor
         * @param for_params just fill parameters description
         */
        UrlCheck(bool for_params)
        {
            WOAP2_LOG_TRACE(" > UrlCheck::UrlCheck")
            WOAP2_LOG_DEBUG(std::string("UrlCheck ") + (for_params ? "for params" : "for check"))
            
            if(for_params)
            {
                check_params.push_back(CheckParameter{url_param_name, "text", 255});
                check_params.push_back(CheckParameter{status_param_name, "longinteger", 600, "200"});
                check_params.push_back(CheckParameter{search_text_param_name, "text", 255});
            }
    
            WOAP2_LOG_TRACE(" < UrlCheck::UrlCheck")
        }
    
        /***
         * Destructor
         */
        virtual ~UrlCheck()
        {
            WOAP2_LOG_TRACE(" > UrlCheck::~UrlCheck")
            WOAP2_LOG_TRACE(" < UrlCheck::~UrlCheck")
        }
    
        /***
         * Get the name of the check
         * @return "URL"
         */
        static std::string get_name()
        {
            return "URL";
        }
    
        /***
         * Get the description of the check
         * @return
         */
        virtual std::string get_description() override
        {
            return "URL status test";
        }
    
        /***
         * Execute check
         * @param params check parameters
         * @return result
         */
        virtual CheckResult execute(ParamsMap const &params) override;
    };
}


#endif //WOAP2_URLCHECK_H
