//
// Created by kirdyk on 21.09.16.
//

#ifndef WOAP2_LOGGER_H
#define WOAP2_LOGGER_H

#include <string>
#include <memory>
#include <cstdio>
#include <ctime>
#include <cstring>
#include <boost/thread.hpp>
#include <sstream>

#ifndef WOAP2_LOG
#define WOAP2_LOG(level, msg) { \
if(logger::Logger::current_logger->get_log_level() >= level)\
    logger::Logger::current_logger->log(level, std::string(msg));\
}
#define WOAP2_LOG_ERROR(msg) WOAP2_LOG(logger::Logger::ERROR, msg) // { logger::Logger::current_logger->log(logger::Logger::ERROR, std::string(msg));}
#define WOAP2_LOG_WARN(msg) WOAP2_LOG(logger::Logger::WARN, msg) //{logger::Logger::current_logger->log(logger::Logger::WARN, std::string(msg));}
#define WOAP2_LOG_INFO(msg) WOAP2_LOG(logger::Logger::INFO, msg) //{logger::Logger::current_logger->log(logger::Logger::INFO, std::string(msg));}
#define WOAP2_LOG_DEBUG(msg) WOAP2_LOG(logger::Logger::DEBUG, msg) // {logger::Logger::current_logger->log(logger::Logger::DEBUG, std::string(msg));}
#define WOAP2_LOG_TRACE(msg) WOAP2_LOG(logger::Logger::TRACE, msg) // {logger::Logger::current_logger->log(logger::Logger::TRACE, std::string(msg));}
#endif

namespace logger
{
    /***
     * Logger base class
     */
    struct Logger
    {
        /***
         * Current logger instance
         */
        static std::shared_ptr<Logger> current_logger;
    
        /***
         * logging levels
         */
        enum LEVEL
        {
            OFF = 0,
            ERROR,
            WARN,
            INFO,
            DEBUG,
            TRACE,
            __LAST__
        };
    
        /***
         * parse level string
         * @param level_string
         * @return
         */
        static LEVEL parse_level(std::string &level_string)
        {
            for(int i = OFF; i < __LAST__; i++)
            {
                if(level_string == level_strings[i])
                {
                    return static_cast<LEVEL>(i);
                }
            }
            
            return ERROR;
        }
    
        /***
         * Constructor
         * @param log_level logging level
         */
        Logger(LEVEL log_level)
        {
            this->log_level = log_level;
        }
        
        virtual ~Logger()
        {}
    
        /***
         * Log message
         * @param level
         * @param msg
         */
        void log(LEVEL level, std::string msg)
        {
            if(level <= log_level)
            {
                time_t _time;
                time(&_time);
                
                char buffer[256];
                
                ctime_r(&_time, buffer);
                
                std::string str_time(buffer, strlen(buffer) - 1);
                
                std::ostringstream oss;
                oss << boost::this_thread::get_id();
                
                std::string message =
                        "[" + str_time + "] " +
                        "(" + oss.str() + ") " +
                        get_level_string(level) + ": " + msg + "\n";
                write_message(message);
            }
        }
    
        /***
         * Log level getter
         * @return
         */
        LEVEL get_log_level() const
        {
            return log_level;
        }
    
        /***
         * Log level setter
         * @param log_level
         */
        void set_log_level(LEVEL log_level)
        {
            Logger::log_level = log_level;
        }
    
    protected:
    
        /***
         * Current log level
         */
        LEVEL log_level;
    
        /***
         * Level strings
         */
        static std::string level_strings[__LAST__];
    
        /***
         * Get string for level.
         * @param level
         * @return
         */
        std::string get_level_string(LEVEL level)
        {
            return level_strings[level];
        }
    
        /***
         * Actual write routine
         * @param msg
         */
        virtual void write_message(std::string &msg) = 0;
    };
    
    /***
     * StrErr logger implementation
     */
    class StderrLogger : public Logger
    {
    public:
        StderrLogger(LEVEL log_level)
                : Logger(log_level)
        {
            
        }
        
        virtual ~StderrLogger()
        {
            
        }
    
    protected:
        virtual void write_message(std::string &msg) override
        {
            fprintf(stderr, "%s", msg.c_str());
            fflush(stderr);
        }
    };
    
    
}
#endif //WOAP2_LOGGER_H
