//
// Created by kirdyk on 06.01.17.
//

#ifndef WOAP2_SNMPCLIENT_H
#define WOAP2_SNMPCLIENT_H

//
// Created by kirdyk on 06.01.17.
//
#include <string>
#include <stdexcept>
#include <cstdint>
#include <mutex>
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include "checks/check.h"
#include "value_comparer.h"
#include "constants.h"

namespace snmp
{
    
    /***
     * SNMP client
     */
    class SnmpClient
    {
        static std::mutex snmp_mutex;
    private:
    
        /***
         * host to check
         */
        std::string host;
        /***
         * SNMP v2c community
         */
        std::string community;
        /***
         * string representation of the value to test with
         */
        std::string test_value_str;
        /***
         * Test operation string
         */
        std::string test_type;
        /***
         * SNMP protocol version
         */
        long proto_version;
    
    
        /***
         * Compare 2 values
         * @param value left hand value
         * @param test_value right hand value
         * @param compare comparison string
         * @return comparison result
         */
        template<typename T>
        bool compare(T value, T test_value, std::string compare)
        {
            // Check comparison type, create the appropriate comparer and do the job
            if(compare == constants::less_string)
            {
                auto c = utils::comparer<T, utils::LESS>();
                return c(value, test_value);
            }
            else if(compare == constants::less_or_equal_string)
            {
                auto c = utils::comparer<T, utils::LESS_OR_EQUAL>();
                return c(value, test_value);
            }
            else if(compare == constants::equal_string)
            {
                auto c = utils::comparer<T, utils::EQUAL>();
                return c(value, test_value);
            }
            else if(compare == constants::not_equal_string)
            {
                auto c = utils::comparer<T, utils::NOT_EQUAL>();
                return c(value, test_value);
            }
            else if(compare == constants::greater_or_equal_string)
            {
                auto c = utils::comparer<T, utils::GREATER_OR_EQUAL>();
                return c(value, test_value);
            }
            else if(compare == constants::greater_string)
            {
                auto c = utils::comparer<T, utils::GREATER>();
                return c(value, test_value);
            }
            else if(compare == constants::contains_string)
            {
                auto c = utils::comparer<T, utils::CONTAINS>();
                return c(value, test_value);
            }
            else if(compare == constants::not_contains_string)
            {
                auto c = utils::comparer<T, utils::NOT_CONTAINS>();
                return c(value, test_value);
            }
            else
            {
                throw std::runtime_error("Unknown test type: " + compare);
            }
        }
    
    public:
        /***
         * Constructor. Create v2c version of the client
         * @param host
         * @param community
         * @param test_value
         * @param test_type
         */
        SnmpClient(std::string &host, std::string &community, std::string &test_value, std::string &test_type)
                : host(host), community(community),
                  test_value_str(test_value), test_type(test_type),
                  proto_version(SNMP_VERSION_2c)
        {
            
        }
    
        /***
         * Get the value and do the comparison.
         * @param oid_name OID name (either literal or numeric)
         * @return check result object
         */
        checks::CheckResult get_value(std::string &oid_name);
    };
}
#endif //WOAP2_SNMPCLIENT_H
