//
// Created by bezgin on 12.09.16.
//


#include "woapworker.h"
#include "utils.h"
#include <sstream>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <sstream>
#include <iomanip>


namespace utils
{
    std::string path_combine(const std::string &p1, const std::string &p2)
    {
        char sep = '/';
        
        if(p1[p1.length()] != sep)
        {
            return p1 + sep + p2;
        }
        else
        {
            return p1 + p2;
        }
        
    }
    
    std::string trim(std::string const &input)
    {
        size_t first = input.find_first_not_of(' ');
        
        first = (first == std::string::npos) ? 0 : first;
    
        size_t last = input.find_last_not_of("\r\n \t");
        
        if(last != std::string::npos)
        {
            return input.substr(first, (last - first + 1));
        }
        else
        {
            return input.substr(first);
        }
    }
    
    
    std::vector<std::string> split(std::string const &input, boost::regex const &re)
    {
        std::vector<std::string> parts;
        
        std::copy(boost::sregex_token_iterator(input.begin(), input.end(), re, -1,
                                               boost::regex_constants::match_not_null), boost::sregex_token_iterator(),
                  std::back_inserter(parts));
        
        return parts;
    }
    
    std::map<std::string, std::string> get_params_map(std::string const &input)
    {
        //http://stackoverflow.com/questions/6462578/alternative-to-regex-match-all-instances-not-inside-quotes
        boost::regex hash_re("#(?=([^\"\\\\]*(\\\\.|\"([^\"\\\\]*\\\\.)*[^\"\\\\]*\"))*[^\"]*$)");
    
        std::vector<std::string> parts = split(input, hash_re);
    
        boost::regex eq_re("=(?=([^\"\\\\]*(\\\\.|\"([^\"\\\\]*\\\\.)*[^\"\\\\]*\"))*[^\"]*$)");
        boost::sregex_token_iterator token_end;
    
        std::map<std::string, std::string> params_map;
    
        for(const std::string &part : parts)
        {
            unsigned long eq_pos = part.find("=");
    
            if(eq_pos != std::string::npos)
            {
                std::string key = part.substr(0, eq_pos);
                std::string value = part.substr(eq_pos + 1);
    
                if(value.length() > 0 && value.at(0) == '"' && value.at(value.length() - 1) == '"')
                {
                    value = value.substr(1, value.length() - 2);
                }
    
                params_map[key] = value;
            }
        }
        return params_map;
    };
    
    std::string params_list_to_string(std::vector<std::pair<std::string, std::string> > const &params_list)
    {
        std::ostringstream out;
        
        for(std::vector<std::pair<std::string, std::string> >::const_iterator param = params_list.begin();
            param != params_list.end(); param++)
        {
            out << "#" << param->first << "=" << param->second;
        }
        
        out << "#";
        
        return out.str();
    }
    
    std::string get_client_ip(const ssh_session &session)
    {
        socket_t sock = ssh_get_fd(session);
        
        struct sockaddr_storage addr;
        socklen_t len = sizeof addr;
        
        getpeername(sock, (sockaddr *) &addr, &len);
        
        char ipstr[INET6_ADDRSTRLEN];
        
        // deal with both IPv4 and IPv6:
        if(addr.ss_family == AF_INET)
        {
            struct sockaddr_in *s = (struct sockaddr_in *) &addr;
            //port = ntohs(s->sin_port);
            inet_ntop(AF_INET, &s->sin_addr, ipstr, sizeof ipstr);
        }
        else
        { // AF_INET6
            struct sockaddr_in6 *s = (struct sockaddr_in6 *) &addr;
            //port = ntohs(s->sin6_port);
            inet_ntop(AF_INET6, &s->sin6_addr, ipstr, sizeof ipstr);
        }
        
        return std::string(ipstr);
    }
    
    /***
     * Compare 2 IPv4 addresses
     * @param addr1
     * @param addr2
     * @return -1 if addr1 < addr2, 0 if equals and 1 if addr1 > addr2
     */
    int compare_ipv4(in_addr addr1, in_addr addr2)
    {
        if(addr1.s_addr < addr2.s_addr)
        {
            return -1;
        }
        else if(addr1.s_addr == addr2.s_addr)
        {
            return 0;
        }
        else
        {
            return 1;
        }
    }
    
    /***
     * Compare 2 IPv6 addresses
     * @param addr1
     * @param addr2
     * @return -1 if addr1 < addr2, 0 if equals and 1 if addr1 > addr2
     */
    int compare_ipv6(in6_addr addr1, in6_addr addr2)
    {
        for(int i = 0; i < 16; i++)
        {
            if(addr1.s6_addr[i] < addr2.s6_addr[i])
            {
                return -1;
            }
            else if(addr1.s6_addr[i] > addr2.s6_addr[i])
            {
                return 1;
            }
        }
        
        return 0;
    }
    
    int compare_addr(sockaddr_storage addr1, sockaddr_storage addr2)
    {
        if(((sockaddr *) &addr1)->sa_family == AF_INET)
        {
            return compare_ipv4(((struct sockaddr_in *) &addr1)->sin_addr, ((struct sockaddr_in *) &addr2)->sin_addr);
        }
        else
        {
            return compare_ipv6(((struct sockaddr_in6 *) &addr1)->sin6_addr,
                                ((struct sockaddr_in6 *) &addr2)->sin6_addr);
        }
    }
    
    
    bool is_address_in_range(const std::string &ip, const std::string &range_begin, const std::string &range_end)
    {
        sockaddr_storage ip_begin, ip_end, ip_test;
        
        if(range_begin.find(':') != std::string::npos) // IPV6
        {
            ip_begin.ss_family = AF_INET6;
            struct sockaddr_in6 *s = (struct sockaddr_in6 *) &ip_begin;
            inet_pton(AF_INET6, range_begin.c_str(), &(s->sin6_addr));
            
            ip_end.ss_family = AF_INET6;
            s = (struct sockaddr_in6 *) &ip_end;
            inet_pton(AF_INET6, range_end.c_str(), &(s->sin6_addr));
            
            ip_test.ss_family = AF_INET6;
            s = (struct sockaddr_in6 *) &ip_test;
            inet_pton(AF_INET6, ip.c_str(), &(s->sin6_addr));
            
            bool result =
                    (compare_ipv6(((struct sockaddr_in6 *) &ip_test)->sin6_addr,
                                  ((struct sockaddr_in6 *) &ip_begin)->sin6_addr) >= 0)
                    && (compare_ipv6(((struct sockaddr_in6 *) &ip_test)->sin6_addr,
                                     ((struct sockaddr_in6 *) &ip_end)->sin6_addr) <= 0);
            return result;
        }
        else
        {
            ip_begin.ss_family = AF_INET;
            struct sockaddr_in *s = (struct sockaddr_in *) &ip_begin;
            inet_pton(AF_INET, range_begin.c_str(), &(s->sin_addr));
            
            ip_end.ss_family = AF_INET;
            s = (struct sockaddr_in *) &ip_end;
            inet_pton(AF_INET, range_end.c_str(), &(s->sin_addr));
            
            ip_test.ss_family = AF_INET;
            s = (struct sockaddr_in *) &ip_test;
            inet_pton(AF_INET, ip.c_str(), &(s->sin_addr));
            
            bool result =
                    (compare_ipv4(((struct sockaddr_in *) &ip_test)->sin_addr,
                                  ((struct sockaddr_in *) &ip_begin)->sin_addr) >= 0)
                    && (compare_ipv4(((struct sockaddr_in *) &ip_test)->sin_addr,
                                     ((struct sockaddr_in *) &ip_end)->sin_addr) <= 0);
            return result;
            
        }
    }
    
    
    std::string to_string(double d, int precision)
    {
        std::ostringstream out;
        
        out << std::fixed << std::setprecision(precision) << d;
        
        return out.str();
    }
}