//
// Created by bezgin on 12.09.16.
//

#ifndef WOAP2_UTILS_H
#define WOAP2_UTILS_H

#include <vector>
#include <string>
#include <map>
#include <algorithm>
#include <boost/regex.hpp>
#include <libssh/libssh.h>

namespace utils
{
    /**
     * Combine two path parts together.
     * @param p1
     * @param p2
     * @return combined path.
     */
    std::string path_combine(const std::string &p1, const std::string &p2);
    
    /**
     * Trim the string. Remove space chars from the beginning and the end of the string.
     * @param input
     * @return trimmed string.
     */
    std::string trim(std::string const &input);
    
    /**
     * Split string by the regex.
     * @param input
     * @param re parts separator regex.
     * @return vector of the string parts.
     */
    std::vector<std::string> split(std::string const &input, boost::regex const &re);
    
    /**
     * Parse command string to command parameters map.
     * @param input
     * @return parsed command parameters map.
     */
    std::map<std::string, std::string> get_params_map(std::string const &input);
    
    /**
     * Compile parameters list to the command string.
     * @param params_list
     * @return compiled command string.
     */
    std::string params_list_to_string(std::vector<std::pair<std::string, std::string>> const &params_list);
    
    /***
     * Get SSH client IP address
     * @param session SSH session
     * @return client IP as a string
     */
    std::string get_client_ip(const ssh_session &session);
    
    /***
     * Check if IP falls in range
     * @param ip
     * @param range_begin
     * @param range_end
     * @return
     */
    bool is_address_in_range(const std::string &ip, const std::string &range_begin, const std::string &range_end);
    
    /***
     * Convert floating-point number to string
     * @param d
     * @param precision
     * @return
     */
    std::string to_string(double d, int precision = 2);
}
#endif //WOAP2_UTILS_H
