//
// Created by kirdyk on 11.01.17.
//

#ifndef WOAP2_VALUE_COMPARER_H
#define WOAP2_VALUE_COMPARER_H

#include <string>
#include <stdexcept>

namespace utils
{
    /***
     * Check types
     */
    enum CHECK
    {
        LESS,
        LESS_OR_EQUAL,
        EQUAL,
        GREATER_OR_EQUAL,
        GREATER,
        NOT_EQUAL,
        CONTAINS,
        NOT_CONTAINS
    };
    
    /***
     * Root empty comparer template
     */
    template<typename T, CHECK check>
    struct comparer
    {
    
    };
    
    /***
     * Generic less comparer
     */
    template<typename T>
    struct comparer<T, LESS>
    {
        bool operator()(const T value, const T test_value) const
        {
            return value < test_value;
        }
    };
    
    /***
     * Generic less or equal comparer
     */
    template<typename T>
    struct comparer<T, LESS_OR_EQUAL>
    {
        bool operator()(const T value, const T test_value) const
        {
            return value <= test_value;
        }
    };
    
    /***
     * Generic equal comparer
     */
    template<typename T>
    struct comparer<T, EQUAL>
    {
        bool operator()(const T value, const T test_value) const
        {
            return value == test_value;
        }
    };
    
    /***
     * Generic not-equal comparer
     */
    template<typename T>
    struct comparer<T, NOT_EQUAL>
    {
        bool operator()(const T value, const T test_value) const
        {
            return value != test_value;
        }
    };
    
    /***
     * Generic greater or equal comparer
     */
    template<typename T>
    struct comparer<T, GREATER_OR_EQUAL>
    {
        bool operator()(const T value, const T test_value) const
        {
            return value >= test_value;
        }
    };
    
    /***
     * Generic greater comparer
     */
    template<typename T>
    struct comparer<T, GREATER>
    {
        bool operator()(const T value, const T test_value) const
        {
            return value > test_value;
        }
    };
    
    /***
     * Generic contains comparer. Will complain about type not supported.
     */
    template<typename T>
    struct comparer<T, CONTAINS>
    {
        bool operator()(const T value, const T test_value) const
        {
            throw std::runtime_error("Can not perform contains check on this data type.");
        }
    };
    
    /***
     * Generic not-contains comparer. Will complain about type not supported.
     */
    template<typename T>
    struct comparer<T, NOT_CONTAINS>
    {
        bool operator()(const T value, const T test_value) const
        {
            throw std::runtime_error("Can not perform contains check on this data type.");
        }
    };
    
    /***
     * String contains comparer
     */
    template<>
    struct comparer<std::string, CONTAINS>
    {
        bool operator()(const std::string &value, const std::string &test_value) const
        {
            return value.find(test_value) != std::string::npos;
        }
    };
    
    /***
     * String not-contains comparer
     */
    template<>
    struct comparer<std::string, NOT_CONTAINS>
    {
        bool operator()(const std::string &value, const std::string &test_value) const
        {
            return value.find(test_value) == std::string::npos;
        }
    };
}

#endif //WOAP2_VALUE_COMPARER_H
