#ifndef WOAPWORKER_H
#define WOAPWORKER_H

#include <memory>
#include <boost/thread.hpp>
#include <libssh/libssh.h>
#include <libssh/server.h>
#include <list>
#include <map>

#include "programoption.h"
#include "commands.h"


/**
 * Main worker class
 */
class WoapWorker
{
    typedef std::unique_ptr<boost::thread> ThreadPtr;
    typedef std::list<ThreadPtr> ThreadList;
    
    /**
     * Stop flag.
     */
    volatile int cancel = 0;
    
    ThreadList threads;
    
    /**
     * Aithenticate user
     * @param user
     * @param password
     * @return true if authenticated.
     */
    bool authenticate(const std::string &user, const std::string &password, const std::string &client_ip);
    
    /**
     * Data exchange thread function.
     * @param session SSH session.
     */
    void thread_func(const ssh_session session);
    
    /**
     * Listen thread function
     * @param sshbind SSH listen bind object.
     */
    void listen_thread_func(const ssh_bind sshbind);
    
    /**
     * Worker thread object.
     */
    std::unique_ptr<boost::thread> worker_thread;
    
    /**
     * Listen port
     */
    int port;
    /**
     * Path to DSA key file.
     */
    std::string dsa_key;
    /**
     * Path to RSA key file
     */
    std::string rsa_key;
    
    /**
     * Passwords map.
     */
    std::shared_ptr<Settings::password_map_type> password_map;
    
    std::shared_ptr<Settings::firewall_map_type> firewall_map;
    
    /**
     * Commands factory object.
     */
    commands::CommandsFactory commands_factory;

public:
    /**
     * Constructor
     * @param settings
     */
    WoapWorker(Settings &settings);
    
    /**
     * Do actual work.
     */
    void do_work();
    
    /**
     * Stop the work.
     */
    void stop()
    {
        cancel = 1;
    }
   
};

#endif // WOAPWORKER_H
